<?php

namespace App\Http\Controllers;

use App\Models\Interacciones;
use App\Models\TipoPersonal;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

class InteraccionesController extends Controller
{
    public function index(Request $request)
    {

        $this->validate($request , [
            'offset' => 'required|integer|min:0' ,
            'limit' => 'required|integer|min:0' ,
            'desde' => 'required|date_format:Y-m-d' ,
            'hasta' => 'required|date_format:Y-m-d' ,
            'fk_idHeader' => 'required' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,
            'offset.required' => ':attribute es requerido' ,
            'limit.required' => ':attribute es requerido' ,
            'desde.required' => ':attribute es requerido' ,
            'hasta.required' => ':attribute es requerido' ,
            'fk_idHeader.required' => ':attribute es requerido' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);

        $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
        $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';

        $todas = Interacciones::orderBy('created_at' , 'asc')
            ->where('fk_idHeader' , $request->fk_idHeader)
            ->whereBetween('created_at' , [$desde , $hasta])
            ->offset($request->offset)
            ->limit($request->limit)
            ->get();

        return response()->json($todas);
    }

    public function actualizarLeido(Request $request)
    {
        $this->validate($request , [
            'idInteraccion' => 'required' ,
            'leido' => 'required|boolean' ,
            'fk_idUser' => 'required' ,
            'fk_idUser_lector' => 'required' ,
        ] , [
            'descripcion.required' => ':attribute es requerido' ,
            'leido.required' => ':attribute es requerido' ,
            'leido.boolean' => ':attribute debe ser 1 o 0 boolean' ,
            'fk_idUser.required' => ':attribute es requerido' ,
            'fk_idUser_lector.required' => ':attribute es requerido' ,
        ]);

        $i = Interacciones::find($request->idInteraccion);
        if ($i){
            $i->update([
                'leido' => (boolean)$request->leido ,
                'fk_idUser' => $request->fk_idUser ,
                'fk_idUser_lector' => $request->fk_idUser_lector
            ]);
            $i->save();
        } else {
            return response()->json(['msj' => 'No actualizó debido a que no existe la interacción'] , 404);
        }
        return response()->json(['msj' => 'Se actualizó exitosamente' , 'interaccion' => $i->refresh()]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'descripcion' => 'required' ,
        ] , [
            'descripcion.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $tipo = new Interacciones($request->all());
            if ($request->exists('file')){
                //obtenemos el campo file definido en el formulario
                $file = $request->file('file');

                //obtenemos el nombre del archivo
                $nombre         = $file->getClientOriginalName();
                $nombre_interno = $nombre . '-' . time();
                //indicamos que queremos guardar un nuevo archivo en el disco local
                Storage::disk('public')->put('\\interacion\\' . $nombre_interno , File::get($file));
                $tipo->file = $nombre_interno;
            }
            $tipo->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'tipo' => $tipo ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en InteraccionesController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = Interacciones::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['interaccion' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'descripcion' => 'required' ,
        ] , [
            'descripcion.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = Interacciones::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir la interaccion solicitada'] , 401);
        }

        try {
            $h->fill($request->all());
            if ($request->exists('file')){
                //obtenemos el campo file definido en el formulario
                $file = $request->file('file');

                //obtenemos el nombre del archivo
                $nombre         = $file->getClientOriginalName();
                $nombre_interno = $nombre . '-' . time();
                //indicamos que queremos guardar un nuevo archivo en el disco local
                Storage::disk('public')->put('\\interacion\\' . $nombre_interno , File::get($file));
                $h->file = $nombre_interno;
            }
            $h->save();
            $response = [
                'msj' => 'Interaccion actualizada exitosamente' ,
                'tipo' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en InteraccionesController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = Interacciones::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }
        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
